// Written by Josh Dersch, summer 2021

#pragma once

#include<cstdarg>
#include<string>
#include<map>
#include<vector>
#include<stack>

struct Environment
{
    Environment* Parent;


};

static wchar_t ConvertGreekChar(char c)
{
    static std::map<char, wchar_t> greekMap =
    {
        { 'Q', L'Θ' },
        { 'W', L'Ω' },
        { 'E', L'Ε' },
        { 'R', L'Ρ' },
        { 'T', L'Τ' },
        { 'Y', L'Ψ' },
        { 'U', L'Υ' },
        { 'I', L'Ι' },
        { 'O', L'Ο' },
        { 'P', L'Π' },
        { 'q', L'θ' },
        { 'w', L'ω' },
        { 'e', L'ε' },
        { 'r', L'ρ' },
        { 't', L'τ' },
        { 'y', L'ψ' },
        { 'u', L'υ' },
        { 'i', L'ι' },
        { 'o', L'ο' },
        { 'p', L'π' },
        { 'A', L'Α' },
        { 'S', L'Σ' },
        { 'D', L'Δ' },
        { 'F', L'Φ' },
        { 'G', L'Γ' },
        { 'H', L'Η' },
        { 'J', L' ' },
        { 'K', L'Κ' },
        { 'L', L'Λ' },
        { 'a', L'α' },
        { 's', L'σ' },
        { 'd', L'δ' },
        { 'f', L'φ' },
        { 'g', L'γ' },
        { 'h', L'η' },
        { 'j', L'ς' },
        { 'k', L'κ' },
        { 'l', L'λ' },
        { 'Z', L'Ζ' },
        { 'X', L'Χ' },
        { 'C', L'Ξ' },
        { 'V', L' ' },
        { 'B', L'Β' },
        { 'N', L'Ν' },
        { 'M', L'Μ' },
        { 'z', L'ζ' },
        { 'x', L'χ' },
        { 'c', L'ξ' },
        { 'v', L' ' },
        { 'b', L'β' },
        { 'n', L'ν' },
        { 'm', L'μ' },
    };

    auto found = greekMap.find(c);
    if (found != greekMap.end())
    {
        return found->second;
    }
    else
    {
        // Return the original character
        return c;
    }
}

static wchar_t ConvertMathChar(char c)
{
    static std::map<char, wchar_t> mathMap =
    {
        { '!', L'†' },
        { '@', L'℅' },
        { '#', L'∞' },
        { '$', L'∑' },
        { '%', L'÷' },
        { '^', L'⇋' },
        { '&', L'∧'},
        { '*', L'∙' },
        { '(', L'`' },
        { ')', L'∏' },
        { '+', L'±' },
        { '{', L'¼' },
        { '}', L'¾' },
        { '1', L'□' },
        { '2', L'∆' },
        { '3', L'∮' },
        { '4', L'⊕' },
        { '5', L'⊖' },
        { '6', L'⊗' },
        { '7', L'∠' },
        { '8', L'⋆' },
        { '9', L'•' },
        { '0', L'○' },
        { '-', L'∓' },
        { '=', L'≠' },
        { '[', L'≺' },
        { ']', L'≻' },
        { 'Q', L'⇄' },
        { 'W', L'{' },
        { 'E', L'∃' },
        { 'R', L']' },
        { 'T', L'⊥' },
        { 'Y', L'↴' },
        { 'U', L'∪' },
        { 'I', L'⊆' },
        { 'O', L'∅' },
        { 'P', L'⧜' },
        // TODO: unsure what the up arrow character maps to
        { 'q', L'⊫' },
        { 'w', L'⪡' },
        { 'e', L'≃' },
        { 'r', L'®' },
        { 't', L'⭦' },
        { 'y', L'∩' },
        { 'u', L'√' },
        { 'i', L'⌐' },
        { 'o', L'∘' },
        { 'p', L'⊢' },
        { '_', L'⭢' },
        { 'A', L'∀' },
        { 'S', L'≈' },
        { 'D', L'∇' },
        { 'F', L'‡' },
        { 'G', L'⊂' },
        { 'H', L'⟮' },
        { 'J', L'⊃' },
        { 'K', L'⊅' },
        { 'L', L'⊇' },
        { ':', L'§' },
        { '"', L'°'},
        { 'a', L'⟨' },
        { 's', L'⭧' },
        { 'd', L'∂' },
        { 'f', L'⟩' },
        { 'g', L'⇒' },
        { 'h', L'ℏ' },
        { 'j', L'⏋' },
        { 'k', L'⏐' },
        { 'l', L'⌙' },
        { ';', L'▮' },
        { '\'', L'≐' },
        { 'Z', L'⏵' },
        { 'X', L'⨉' },
        { 'C', L'⟯' },
        { 'V', L'∨' },
        { 'B', L'∈' },
        { 'N', L'∉' },
        { 'M', L'[' },
        { '<', L'≤' },
        { '>', L'≥' },
        { '?', L'¿' },
        { 'z', L'⊣' },
        { 'x', L'⪢' },
        { 'c', L'©' },
        { 'v', L'⭨' },
        { 'b', L'↝' },
        { 'n', L'¬' },
        { 'm', L'∥' },
        { ',', L'∋'},
        { '.', L'∫' },
        { '/', L'⊘' },
        { ' ', L'␣' }
    };

    auto found = mathMap.find(c);
    if (found != mathMap.end())
    {
        return found->second;
    }
    else
    {
        // Return the "replacement character" code.
        return static_cast<wchar_t>(0xfffd);
    }
}

class ITiogaOutput
{
public:
    ITiogaOutput(FILE* stream) { m_outputStream = stream; }
    virtual ~ITiogaOutput() { };

    virtual void BeginOutput(const char* title) = 0;
    virtual void EndOutput() = 0;

    virtual void StartNode(tread_Reader* reader, const char* format, const char* comment)
    {
        m_currentLooks.clear();
    }

    virtual void EndNode(tread_Reader* reader) = 0;
    virtual void AddLook(tread_Reader* reader, long looks, long start, int len)
    {
        if (m_currentLooks.size() < start + len)
        {
            m_currentLooks.resize(start + len);
        }

        for (unsigned i = start; i < start + len; i++)
        {
            m_currentLooks[i] |= looks;
        }
    }

    virtual void InsertText(tread_Reader* reader, const unsigned char* text, long len, int comment) = 0;
    virtual void HandleProp(tread_Reader* reader, const char* propName, const unsigned char* prop, long len) = 0;

    virtual void BeginLook(uint32_t look) = 0;
    virtual void EndLook(uint32_t look) = 0;

protected:
    void ChangeLook(uint32_t lastLook, uint32_t newLook)
    {
        // Walk the bits and see what bits have been removed from newLook, output terminating markup as appropriate.
        // Then Walk the bits and see what bits have been added to newLook, output markup as appropriate.
        for (int look = 0; look < 32; look++)
        {
            if ((lastLook & (1 << look)) && !(newLook & (1 << look)))
            {
                EndLook(31 - look);
            }
        }

        for (int look = 0; look < 32; look++)
        {
            if (!(lastLook & (1 << look)) && (newLook & (1 << look)))
            {
                BeginLook(31 - look);
            }
        }
    }

    void WriteOutput(const wchar_t* format, ...) {
        va_list argp;
        va_start(argp, format);

        vfwprintf(m_outputStream, format, argp);
    }

    FILE* m_outputStream;
    std::vector<uint32_t> m_currentLooks;

};

class TiogaOutputText : public ITiogaOutput
{
public:
    TiogaOutputText(FILE* stream, unsigned columnWidth);
    ~TiogaOutputText();

    virtual void BeginOutput(const char* title) { };
    virtual void EndOutput() { };

    virtual void StartNode(tread_Reader* reader, const char* format, const char* comment);
    virtual void EndNode(tread_Reader* reader);
    virtual void InsertText(tread_Reader* reader, const unsigned char* text, long len, int comment);
    virtual void HandleProp(tread_Reader* reader, const char* propName, const unsigned char* prop, long len);

private:
    virtual void BeginLook(uint32_t look) { };
    virtual void EndLook(uint32_t look) { };

    void WriteIndent();

    unsigned m_columnWidth;

    //
    // This is kind of a hack -- since the only node format we can really do anything with involves
    // indents we just keep a stack of integers here representing the indentation for the specified node
    // (in characters).  Not all nodes affect the indent; those have an indent of "0".
    // m_currentIndent tracks the current sum of all applicable indents.
    //
    std::stack<unsigned> m_indentStack;
    unsigned m_currentIndent;
};

class TiogaOutputHtml : public ITiogaOutput
{
public:
    TiogaOutputHtml(FILE* stream);
    ~TiogaOutputHtml();

    virtual void BeginOutput(const char* title);
    virtual void EndOutput() { WriteOutput(L"</body>\n</html>\n"); }

    virtual void StartNode(tread_Reader* reader, const char* format, const char* comment);
    virtual void EndNode(tread_Reader* reader);
    virtual void InsertText(tread_Reader* reader, const unsigned char* text, long len, int comment);
    virtual void HandleProp(tread_Reader* reader, const char* propName, const unsigned char* prop, long len);

private:
    virtual void BeginLook(uint32_t look);
    virtual void EndLook(uint32_t look);

};

int tioga_to_text_or_html(const char* ifs_file_path, // only for HTML title
                          bool textOutput,
                          const char* in_path,
                          const char* out_path);
