/* Copyright (c) Xerox Corporation 1993. All rights reserved. */
/* tread.h -- Low-level Tioga reader routines.
   David Nichols
   December, 1990 */

struct tread_Stream 
{
    unsigned char *next;
    unsigned char *limit;
};

struct tread_Reader;

struct tread_Procs 
{
    int (*startNode)(tread_Reader *reader, const char *format, const char *comment);
    int (*endNode)(tread_Reader *reader);
    int (*addLooks)(tread_Reader *reader, long looks, long start, int len);
    int (*insertText)(tread_Reader *reader, const unsigned char *text, long len, int comment);
    int (*handleProp)(tread_Reader *reader, const char *propName, const unsigned char *prop, long len);
   
};

struct tread_Reader 
{
    tread_Procs *procs;
    void *data;			/* private data for client */
    /* Full buffer. */
    char *buf;
    long totalLen;
    /* Start and end of each region. */
    tread_Stream text;
    tread_Stream com;
    tread_Stream control;
    /* Generic piece of text */
    int strLen;
    unsigned char *str;
    /* Formats */
    int nFormats;		/* number in use */
    char *formats[tioga_NumFormats]; /* list of them */
    /* Look combos */
    int nLooks;			/* number in use */
    long looks[tioga_NumLooks];	/* list of them */
    /* Property names */
    int nProps;
    char *props[tioga_NumProps];
};

/* Initialize from buffer holding entire file. */
int tread_Init(
    tread_Reader *r, 
    unsigned char *buf, 
    long len,
    tread_Procs *procs, 
    void *data);

/* Get next opcode from control stream. */
tioga_ControlOp tread_GetOp(tread_Reader *r);

/* Get an byte from the control stream. */
unsigned char tread_GetByte(tread_Reader* r);

/* Get an integer from the control stream. */
long tread_GetInt(tread_Reader *r);

/* Get text from control stream. */
int tread_GetStr(tread_Reader *r);

/* Get text from a specific stream. */
int tread_SGetRope(
    tread_Reader *r,
    tread_Stream *s, 
    long len);

/* Add a new format, and return its index. */
int tread_AddFormat(
    tread_Reader*r,
    unsigned char *format);

/* Add a new looks vector, return its index. */
int tread_AddLooks(
    tread_Reader*r,
    long l);

/* Add a new property name, return its index. */
int tread_AddProp(
    tread_Reader *r, 
    const char *propName);
